import os
import shutil
import argparse
from hops import hdfs
from pyspark.sql import SparkSession


def setup_spark() -> SparkSession:
    return SparkSession.builder.enableHiveSupport().getOrCreate()


def get_dest(src, dest):
    if not dest:
        return os.path.abspath(os.path.join(src, os.pardir))
    return dest


def unpack_dir(filename):
    for _, info in shutil._UNPACK_FORMATS.items():
        for extension in info[0]:
            if filename.endswith(extension):
                return filename.replace(extension, "")
    raise shutil.ReadError("Unknown archive format '{0}'".format(filename))


def compress(src, dest, format, overwrite):
    file_name = os.path.basename(src)
    hdfs.copy_to_local(src)
    compressed = shutil.make_archive(file_name, format, base_dir=file_name)
    hdfs.copy_to_hdfs(compressed, get_dest(src, dest), overwrite)


def extract(src, dest, overwrite):
    file_name = os.path.basename(src)
    extract_dir = unpack_dir(file_name)
    hdfs.copy_to_local(src)
    shutil.unpack_archive(file_name, extract_dir)
 
    dest = get_dest(src, dest)
    dest_dir = os.path.join(dest, extract_dir)

    # if dest is an empty dir overwrite. For backward compatibility 
    if hdfs.exists(dest_dir) and len(hdfs.ls(dest_dir)) == 0:
        overwrite = True

    # To create extract_dir even for archives without dir entry but avoid 
    # extract_dir/extract_dir for archives with dir entry
    extract_path = os.path.join(extract_dir, extract_dir)
    if os.path.exists(extract_path) and len(os.listdir(extract_dir)) == 1:
        extract_dir = extract_path

    hdfs.copy_to_hdfs(extract_dir, get_dest(src, dest), overwrite)


if __name__ == "__main__":
    # Setup spark first so it fails faster in case of args errors
    # Otherwise the resource manager will wait until the spark application master
    # registers, which never happens.
    spark = setup_spark()
    format_choices = [i[0] for i in shutil.get_archive_formats()]

    parser = argparse.ArgumentParser(description="Hops file operation Job")
    parser.add_argument(
        "-op",
        type=str,
        choices=[
            "compress",
            "extract"
        ],
        required=True,
        help="Operation type",
    )
    parser.add_argument(
        "-src",
        nargs='+',
        type=str,
        required=True,
        help="Location on HopsFS of the source file to compress/extract",
    )
    parser.add_argument(
        "-dest",
        nargs='+',
        type=str,
        help="Location on HopsFS of the destination path for the compressed/extracted file",
    )
    parser.add_argument(
        "-format",
        type=str,
        default="zip",
        choices=format_choices,
        help="The archive format",
    )
    parser.add_argument(
        "-overwrite",
        type=bool,
        default=False,
        help="Overwrite the destination directory if it exists",
    )

    args = parser.parse_args()

    # fix yarn runner not allowing space in args. 
    # src='/Projects/project1/testDataset/test dir/sub dir' will be received as
    # src='/Projects/project1/testDataset/test --arg dir/sub --arg dir' from 
    # https://github.com/logicalclocks/hopsworks-ee/blob/master/hopsworks-common/src/main/java/io/hops/hopsworks/common/jobs/yarn/YarnRunner.java#L219
    dest = None
    if args.dest: dest = " ".join(args.dest)
    src = " ".join(args.src)

    if args.op == "compress":
        compress(src, dest, args.format, args.overwrite)
    elif args.op == "extract":
        extract(src, dest, args.overwrite)
